import requests
from bs4 import BeautifulSoup
import pandas as pd
import re


def validate_url(url):
    pattern = r'https?://www\.nike.com/.*'
    if not re.match(pattern, url):
        raise ValueError('Invalid URL: Must be a valid Nike.com URL.')


def request_page(url):
    page = requests.get(url)
    soup = BeautifulSoup(page.content, 'html.parser')
    return soup


def get_nike_products(url):
    """
    This function takes a Nike.com URL and returns a dataframe.

    Args:
         url (:obj:`str`): String Nike.com url

    Returns:
        data: A panda Dataframe with various information regarding Nike products.
    """
    validate_url(url)
    soup = request_page(url)
    products_info = []
    products = soup.find_all('div', {'class': 'product-card__body'})
    for product in products:
        name = product.find('div', {'class': 'product-card__title'}).text.strip()
        price = product.find('div', {'class': 'product-price'}).text.strip()
        type = product.find('div', {'class': 'product-card__subtitle'}).text.strip()
        product_url = product.find('a').get('href')
        description = get_product_description(product_url)
        data = {
            'Product Name': name,
            'Price': price,
            'Type': type,
            'Description': description,
            'Product URL': product_url,
        }
        products_info.append(data)
    data = pd.DataFrame(products_info)
    return data


def sort_nike_products(products_info, sort_order='asc'):
    """
    This function sorts Nike dataframe based on price. Can be sorted in ascending or descending order.
    Args:
         products_info (:obj: `dataframe`): Nike products dataframe
         sort_order (:obj: `str`): Ascending or descending order

    Returns:
        data: A panda Dataframe in ascending or descending order.
    """
    if sort_order == 'asc':
        return products_info.sort_values(
            'Price', ascending=True, key=lambda val: val.str.replace('$', '').astype('float64'), ignore_index=True
        )
    elif sort_order == 'desc':
        return products_info.sort_values(
            'Price', ascending=False, key=lambda val: val.str.replace('$', '').astype('float64'), ignore_index=True
        )
    else:
        raise ValueError('Invalid sort order: Must be "asc" or "desc".')


def get_product_description(product_url):
    """
    This function returns a specific product's description.
    Args:
         product_url (:obj: `str`): Nike product's url

    Returns:
         description: A string description.
    """
    validate_url(product_url)
    soup = request_page(product_url)
    description = soup.find('div', {'class': 'description-preview body-2 css-1pbvugb'}).text.strip()
    return description


def filter_nike_products(products_info, price_range=None, product_type=None):
    """
    This function returns filtered nike product dataframe
    Args:
         products_info (:obj: `dataframe`): Nike products dataframe
         price_range (:obj: set): Price range
         product_type (:obj: str): Product type

    Returns:
         filtered_data: A filtered dataframe.
    """
    filtered_data = products_info.copy()

    if price_range:
        min_price, max_price = price_range
        filtered_data['Price'] = filtered_data['Price'].str.replace('$', '').astype(float)
        filtered_data = filtered_data.loc[(filtered_data['Price'] >= min_price) & (filtered_data['Price'] <= max_price)]

    if product_type:
        filtered_data = filtered_data.loc[filtered_data['Type'] == product_type]

    if filtered_data.empty:
        raise ValueError("No products found for the specified criteria.")

    filtered_data['Price'] = filtered_data['Price'].astype(str)
    return filtered_data



from nikefy import (
    request_page,
    validate_url,
    get_nike_products,
    sort_nike_products,
    get_product_description,
    filter_nike_products,
)
from unittest.mock import patch, mock_open, Mock, call
import unittest
from bs4 import BeautifulSoup
import pandas as pd
from pandas.testing import assert_frame_equal


class TestNikefy(unittest.TestCase):
    def test_validate_url_valid(self):
        url = 'https://www.nike.com/'
        self.assertIsNone(validate_url(url))

    def test_validate_url_invalid(self):
        url = 'https://www.adidas.com/'
        with self.assertRaises(ValueError):
            validate_url(url)

    def test_sort_nike_products_asc(self):
        products_info = pd.DataFrame(
            {
                'Product Name': ['Nike Alphafly 2', 'Nike Vaporfly 3', 'Jordan Retro 6 G NRG'],
                'Price': ['$275', '$250', '$220'],
            }
        )
        expected_output = pd.DataFrame(
            {
                'Product Name': ['Jordan Retro 6 G NRG', 'Nike Vaporfly 3', 'Nike Alphafly 2'],
                'Price': ['$220', '$250', '$275'],
            }
        )
        self.assertTrue(expected_output.equals(sort_nike_products(products_info, 'asc')))

    def test_sort_nike_products_desc(self):
        products_info = pd.DataFrame(
            {
                'Product Name': ['Nike Air Max 270', 'Nike Air Max 95', "Nike Air Force 1 '07"],
                'Price': ['$160', '$175', '$110'],
            }
        )
        expected_output = pd.DataFrame(
            {
                'Product Name': ['Nike Air Max 95', 'Nike Air Max 270', "Nike Air Force 1 '07"],
                'Price': ['$175', '$160', '$110'],
            }
        )
        print((sort_nike_products(products_info, 'desc')))
        self.assertTrue(expected_output.equals(sort_nike_products(products_info, 'desc')))

    def test_sort_nike_products_invalid_order(self):
        products_info = pd.DataFrame(
            {
                'Product Name': ['Nike Air Max 270', 'Nike Air Max 95', "Nike Air Force 1 '07"],
                'Price': ['$160', '$175', '$110'],
            }
        )
        with self.assertRaises(ValueError):
            sort_nike_products(products_info, 'invalid')

    def test_get_product_description(self):
        with patch('nikefy.request_page') as mock_request_page:
            mock_request_page.return_value = BeautifulSoup(
                '<div class="description-preview body-2 '
                'css-1pbvugb"><p>With maximum cushioning to support every mile, the Invincible 3 gives you our '
                'highest level of comfort underfoot to help you stay on your feet today, tomorrow and beyond. '
                'Designed to help keep you on the run, it’s super supportive and bouncy, so that you can propel down '
                'your preferred path and come back for your next run feeling ready and reinvigorated.</p></div>',
                'html.parser',
            )
            description = get_product_description(
                'https://www.nike.com/t/invincible-3-mens-road-running-shoes-CLdFjq/DR2615-101'
            )
            self.assertEqual(
                description,
                'With maximum cushioning to support every mile, the Invincible 3 '
                'gives you our highest level of comfort underfoot to help you stay '
                'on your feet today, tomorrow and beyond. Designed to help keep you '
                'on the run, it’s super supportive and bouncy, so that you can propel '
                'down your preferred path and come back for your next run feeling ready '
                'and reinvigorated.'
                'Shown: White/Sail/Oatmeal/Obsidian'
                'Style: DR2615-101',
            )

    def test_get_nike_products_integration(self):
        url = 'https://www.nike.com/w/mens-shoes-nik1zy7ok'
        data = get_nike_products(url)
        self.assertIsInstance(data, pd.DataFrame)
        self.assertGreater(len(data), 0)

    def test_filter_nike_products(self):
        data = {
            'Name': ['Nike Air Max 90', 'Nike Air Force 1', "Nike Blazer Mid '77"],
            'Type': ["Men's Shoes", "Women's Shoes", "Men's Shoes"],
            'Price': ['$130', '$120', '$90'],
        }
        products_info = pd.DataFrame(data)

        filtered_data = filter_nike_products(products_info, price_range=(100, 130), product_type="Men's Shoes")
        expected_data = pd.DataFrame({'Name': ['Nike Air Max 90'], 'Type': ["Men's Shoes"], 'Price': ['130.0']})

        assert_frame_equal(filtered_data, expected_data)

    def test_filter_nike_products_invalid_price_range(self):
        data = {
            'Name': ['Nike Air Max 90', 'Nike Air Force 1', "Nike Blazer Mid '77"],
            'Type': ["Men's Shoes", "Women's Shoes", "Men's Shoes"],
            'Price': ['$130', '$120', '$90'],
        }
        products_info = pd.DataFrame(data)

        with self.assertRaises(ValueError):
            filter_nike_products(products_info, price_range=(150, 200))

    def test_filter_nike_products_invalid_product_type(self):
        data = {
            'Name': ['Nike Air Max 90', 'Nike Air Force 1', "Nike Blazer Mid '77"],
            'Type': ["Men's Shoes", "Women's Shoes", "Men's Shoes"],
            'Price': ['$130', '$120', '$90'],
        }
        products_info = pd.DataFrame(data)

        with self.assertRaises(ValueError):
            filter_nike_products(products_info, product_type="Kids' Shoes")

    def test_filter_nike_products_no_results(self):
        data = {
            'Name': ['Nike Air Max 90', 'Nike Air Force 1', "Nike Blazer Mid '77"],
            'Type': ["Men's Shoes", "Women's Shoes", "Men's Shoes"],
            'Price': ['$130', '$120', '$90'],
        }
        products_info = pd.DataFrame(data)

        with self.assertRaises(ValueError) as cm:
            filter_nike_products(products_info, price_range=(200, 300), product_type="Women's Shoes")

        self.assertEqual(str(cm.exception), "No products found for the specified criteria.")


if __name__ == '__main__':
    unittest.main()



from .nikefy import get_nike_products, sort_nike_products, filter_nike_products

if __name__ == "__main__":
    url = 'https://www.nike.com/w/mens-shoes-nik1zy7ok'
    nike_products = get_nike_products(url)
    print(nike_products)
    sorted_nike_products = sort_nike_products(nike_products, sort_order='asc')
    print(sorted_nike_products)
    filtered_nike_products = filter_nike_products(nike_products, price_range=(100, 150), product_type="Men's Shoes")
    print(filtered_nike_products)
